@@ -10,8 +10,11 @@ from edx_rest_api_client.exceptions import HttpClientError, HttpServerError
 
 from django import forms
 from django.conf import settings
+from django.contrib.auth.models import User
 from django.core.exceptions import ObjectDoesNotExist, ValidationError
+from django.core.urlresolvers import reverse
 from django.db.models.fields import BLANK_CHOICE_DASH
+from django.utils.safestring import mark_safe
 from django.utils.translation import ugettext as _
 
 from enterprise import utils
@@ -25,9 +28,18 @@ from enterprise.admin.utils import (
 )
 from enterprise.api_client.discovery import CourseCatalogApiClient
 from enterprise.api_client.lms import EnrollmentApiClient
-from enterprise.models import EnterpriseCustomer, EnterpriseCustomerIdentityProvider
+from enterprise.models import (
+    EnterpriseCustomer,
+    EnterpriseCustomerIdentityProvider,
+    EnterpriseCustomerReportingConfiguration,
+)
 from enterprise.utils import MultipleProgramMatchError
 
+try:
+    from third_party_auth.models import SAMLProviderConfig as saml_provider_configuration
+except ImportError:
+    saml_provider_configuration = None
+
 logger = getLogger(__name__)  # pylint: disable=invalid-name
 
 
@@ -118,13 +130,16 @@ class ManageLearnersForm(forms.Form):
 
     def __init__(self, *args, **kwargs):
         """
-        Initializes form: puts current user into a field for later access.
+        Initializes form: puts current user and enterprise_customer into a
+        field for later access.
 
         Arguments:
             user (django.contrib.auth.models.User): current user
+            enterprise_customer (enterprise.models.EnterpriseCustomer): current customer
         """
         user = kwargs.pop('user', None)
         self._user = user
+        self._enterprise_customer = kwargs.pop('enterprise_customer', None)
         super(ManageLearnersForm, self).__init__(*args, **kwargs)
 
     def clean_email_or_username(self):
@@ -188,7 +203,7 @@ class ManageLearnersForm(forms.Form):
             return None
 
         try:
-            client = CourseCatalogApiClient(self._user)
+            client = CourseCatalogApiClient(self._user, self._enterprise_customer.site)
             program = client.get_program_by_uuid(program_id) or client.get_program_by_title(program_id)
         except MultipleProgramMatchError as exc:
             raise ValidationError(ValidationMessages.MULTIPLE_PROGRAM_MATCH.format(program_count=exc.programs_matched))
@@ -275,7 +290,7 @@ class ManageLearnersForm(forms.Form):
 
         course_runs = get_course_runs_from_program(program)
         try:
-            client = CourseCatalogApiClient(self._user)
+            client = CourseCatalogApiClient(self._user, self._enterprise_customer.site)
             available_modes = client.get_common_course_modes(course_runs)
             course_mode = self.cleaned_data.get(self.Fields.COURSE_MODE)
         except (HttpClientError, HttpServerError):
@@ -299,7 +314,18 @@ class EnterpriseCustomerAdminForm(forms.ModelForm):
     """
     class Meta:
         model = EnterpriseCustomer
-        fields = "__all__"
+        fields = (
+            "name",
+            "active",
+            "site",
+            "catalog",
+            "enable_data_sharing_consent",
+            "enforce_data_sharing_consent",
+            "enable_audit_enrollment",
+            "enable_audit_data_reporting",
+            "replace_sensitive_sso_username",
+            "hide_course_original_price",
+        )
 
     class Media:
         js = ('enterprise/admin/enterprise_customer.js', )
@@ -316,10 +342,13 @@ class EnterpriseCustomerAdminForm(forms.ModelForm):
         self.fields['catalog'] = forms.ChoiceField(
             choices=self.get_catalog_options(),
             required=False,
-            help_text="<a id='catalog-details-link' href='#' target='_blank'"
-                      "data-url-template='{catalog_admin_url}'> View catalog details.</a>".format(
-                          catalog_admin_url=utils.get_catalog_admin_url_template(),
-                      )
+            # pylint: disable=bad-continuation
+            help_text='<a id="catalog-details-link" href="#" target="_blank" '
+                      'data-url-template="{catalog_admin_change_url}"> View catalog details.</a>'
+                      ' <p style="margin-top:-4px;"><a href="{catalog_admin_add_url}"'
+                      ' target="_blank">Create a new catalog</a></p>'.format(
+                catalog_admin_change_url=utils.get_catalog_admin_url_template(mode='change'),
+                catalog_admin_add_url=utils.get_catalog_admin_url_template(mode='add'))
         )
 
     def get_catalog_options(self):
@@ -329,7 +358,16 @@ class EnterpriseCustomerAdminForm(forms.ModelForm):
         Once retrieved, these name pairs can be used directly as a value
         for the `choices` argument to a ChoiceField.
         """
-        catalog_api = CourseCatalogApiClient(self.user)
+        # TODO: We will remove the discovery service catalog implementation
+        # once we have fully migrated customer's to EnterpriseCustomerCatalogs.
+        # For now, this code will prevent an admin from creating a new
+        # EnterpriseCustomer with a discovery service catalog. They will have to first
+        # save the EnterpriseCustomer admin form and then edit the EnterpriseCustomer
+        # to add a discovery service catalog.
+        if hasattr(self.instance, 'site'):
+            catalog_api = CourseCatalogApiClient(self.user, self.instance.site)
+        else:
+            catalog_api = CourseCatalogApiClient(self.user)
         catalogs = catalog_api.get_all_catalogs()
         # order catalogs by name.
         catalogs = sorted(catalogs, key=lambda catalog: catalog.get('name', '').lower())
@@ -371,8 +409,27 @@ class EnterpriseCustomerIdentityProviderAdminForm(forms.ModelForm):
         """
         super(EnterpriseCustomerIdentityProviderAdminForm, self).__init__(*args, **kwargs)
         idp_choices = utils.get_idp_choices()
+        help_text = ''
+        if saml_provider_configuration:
+            provider_id = self.instance.provider_id
+            url = reverse('admin:{}_{}_add'.format(
+                saml_provider_configuration._meta.app_label,
+                saml_provider_configuration._meta.model_name))
+            if provider_id:
+                identity_provider = utils.get_identity_provider(provider_id)
+                update_url = url + '?source={}'.format(identity_provider.pk)
+                help_text = '<p><a href="{update_url}" target="_blank">View "{identity_provider}" details</a><p>'.\
+                    format(update_url=update_url, identity_provider=identity_provider.name)
+
+            help_text += '<p style="margin-top:-5px;"><a target="_blank" href={add_url}>' \
+                         'Create a new identity provider</a></p>'.format(add_url=url)
+
         if idp_choices is not None:
-            self.fields['provider_id'] = forms.TypedChoiceField(choices=idp_choices)
+            self.fields['provider_id'] = forms.TypedChoiceField(
+                choices=idp_choices,
+                label=_('Identity Provider'),
+                help_text=mark_safe(help_text),
+            )
 
     def clean(self):
         """
@@ -416,3 +473,66 @@ class EnterpriseCustomerIdentityProviderAdminForm(forms.ModelForm):
                     identity_provider_site=identity_provider.site,
                 ),
             )
+
+
+class EnterpriseCustomerReportingConfigAdminForm(forms.ModelForm):
+    """
+    Alternate form for the EnterpriseCustomerReportingConfiguration admin page.
+
+    This form uses the PasswordInput widget to obscure passwords as they are
+    being entered by the user.
+    """
+    class Meta:
+        model = EnterpriseCustomerReportingConfiguration
+        fields = (
+            "enterprise_customer",
+            "active",
+            "data_type",
+            "report_type",
+            "delivery_method",
+            "frequency",
+            "day_of_month",
+            "day_of_week",
+            "hour_of_day",
+            "email",
+            "decrypted_password",
+            "sftp_hostname",
+            "sftp_port",
+            "sftp_username",
+            "decrypted_sftp_password",
+            "sftp_file_path",
+        )
+        widgets = {
+            'decrypted_password': forms.widgets.PasswordInput(),
+            'decrypted_sftp_password': forms.widgets.PasswordInput(),
+        }
+
+
+class TransmitEnterpriseCoursesForm(forms.Form):
+    """
+    Form to transmit courses metadata for enterprise customers.
+    """
+    channel_worker_username = forms.CharField(
+        label=_('Enter enterprise channel worker username.'),
+        required=True
+    )
+
+    def clean_channel_worker_username(self):
+        """
+        Clean enterprise channel worker user form field
+
+        Returns:
+            str: the cleaned value of channel user username for transmitting courses metadata.
+        """
+        channel_worker_username = self.cleaned_data['channel_worker_username'].strip()
+
+        try:
+            User.objects.get(username=channel_worker_username)
+        except User.DoesNotExist:
+            raise ValidationError(
+                ValidationMessages.INVALID_CHANNEL_WORKER.format(
+                    channel_worker_username=channel_worker_username
+                )
+            )
+
+        return channel_worker_username
