@@ -16,6 +16,8 @@ from django.contrib.auth import get_permission_codename
 from django.contrib.auth.models import User
 from django.core import mail
 from django.core.exceptions import ValidationError
+from django.core.management import call_command
+from django.core.urlresolvers import reverse
 from django.db.models import Q
 from django.http import HttpResponse, HttpResponseRedirect
 from django.shortcuts import get_object_or_404, render
@@ -24,8 +26,9 @@ from django.utils.translation import ugettext as _
 from django.utils.translation import ungettext
 from django.views.generic import View
 
-from enterprise.admin.forms import ManageLearnersForm
+from enterprise.admin.forms import ManageLearnersForm, TransmitEnterpriseCoursesForm
 from enterprise.admin.utils import (
+    UrlNames,
     ValidationMessages,
     email_or_username__to__email,
     get_course_runs_from_program,
@@ -34,17 +37,15 @@ from enterprise.admin.utils import (
     split_usernames_and_emails,
     validate_email_to_link,
 )
-from enterprise.api_client.discovery import CourseCatalogApiClient
-from enterprise.api_client.lms import EnrollmentApiClient, parse_lms_api_datetime
+from enterprise.api_client.lms import EnrollmentApiClient
 from enterprise.models import (
     EnrollmentNotificationEmailTemplate,
     EnterpriseCourseEnrollment,
     EnterpriseCustomer,
     EnterpriseCustomerUser,
-    PendingEnrollment,
     PendingEnterpriseCustomerUser,
 )
-from enterprise.utils import get_configuration_value_for_site, send_email_notification_message
+from enterprise.utils import get_configuration_value_for_site, send_email_notification_message, track_enrollment
 
 
 class TemplatePreviewView(View):
@@ -92,6 +93,94 @@ class TemplatePreviewView(View):
         return request.user.first_name or request.user.username
 
 
+class EnterpriseCustomerTransmitCoursesView(View):
+    """
+    Transmit courses view.
+
+    Allows transmitting of courses metadata for provided enterprise.
+    """
+    template = 'enterprise/admin/transmit_courses_metadata.html'
+
+    class ContextParameters(object):
+        """
+        Namespace-style class for custom context parameters.
+        """
+        ENTERPRISE_CUSTOMER = 'enterprise_customer'
+        TRANSMIT_COURSES_METADATA_FORM = 'transmit_courses_metadata_form'
+
+    @staticmethod
+    def _build_admin_context(request, customer):
+        """
+        Build common admin context.
+        """
+        opts = customer._meta
+        codename = get_permission_codename('change', opts)
+        has_change_permission = request.user.has_perm('%s.%s' % (opts.app_label, codename))
+        return {
+            'has_change_permission': has_change_permission,
+            'opts': opts
+        }
+
+    def _build_context(self, request, enterprise_customer_uuid):
+        """
+        Build common context parts used by different handlers in this view.
+        """
+        enterprise_customer = EnterpriseCustomer.objects.get(uuid=enterprise_customer_uuid)  # pylint: disable=no-member
+
+        context = {
+            self.ContextParameters.ENTERPRISE_CUSTOMER: enterprise_customer,
+        }
+        context.update(admin.site.each_context(request))
+        context.update(self._build_admin_context(request, enterprise_customer))
+        return context
+
+    def get(self, request, enterprise_customer_uuid):
+        """
+        Handle GET request - render "Transmit courses metadata" form.
+
+        Arguments:
+            request (django.http.request.HttpRequest): Request instance
+            enterprise_customer_uuid (str): Enterprise Customer UUID
+
+        Returns:
+            django.http.response.HttpResponse: HttpResponse
+        """
+        context = self._build_context(request, enterprise_customer_uuid)
+        transmit_courses_metadata_form = TransmitEnterpriseCoursesForm()
+        context.update({self.ContextParameters.TRANSMIT_COURSES_METADATA_FORM: transmit_courses_metadata_form})
+
+        return render(request, self.template, context)
+
+    def post(self, request, enterprise_customer_uuid):
+        """
+        Handle POST request - handle form submissions.
+
+        Arguments:
+            request (django.http.request.HttpRequest): Request instance
+            enterprise_customer_uuid (str): Enterprise Customer UUID
+        """
+        transmit_courses_metadata_form = TransmitEnterpriseCoursesForm(request.POST)
+
+        # check that form data is well-formed
+        if transmit_courses_metadata_form.is_valid():
+            channel_worker_username = transmit_courses_metadata_form.cleaned_data['channel_worker_username']
+
+            # call `transmit_content_metadata` management command to trigger
+            # transmission of enterprise courses metadata
+            call_command(
+                'transmit_content_metadata',
+                '--catalog_user', channel_worker_username,
+                enterprise_customer=enterprise_customer_uuid
+            )
+
+            # Redirect to GET
+            return HttpResponseRedirect('')
+
+        context = self._build_context(request, enterprise_customer_uuid)
+        context.update({self.ContextParameters.TRANSMIT_COURSES_METADATA_FORM: transmit_courses_metadata_form})
+        return render(request, self.template, context)
+
+
 class EnterpriseCustomerManageLearnersView(View):
     """
     Manage Learners view.
@@ -140,9 +229,7 @@ class EnterpriseCustomerManageLearnersView(View):
             self.ContextParameters.PENDING_LEARNERS: pending_linked_learners,
             self.ContextParameters.LEARNERS: linked_learners,
             self.ContextParameters.SEARCH_KEYWORD: search_keyword or '',
-            self.ContextParameters.ENROLLMENT_URL: getattr(settings,
-                                                           'ENTERPRISE_PUBLIC_ENROLLMENT_API_URL',
-                                                           settings.ENTERPRISE_ENROLLMENT_API_URL),
+            self.ContextParameters.ENROLLMENT_URL: settings.LMS_ENROLLMENT_API_PATH,
         }
         context.update(admin.site.each_context(request))
         context.update(self._build_admin_context(request, enterprise_customer))
@@ -163,9 +250,12 @@ class EnterpriseCustomerManageLearnersView(View):
             customer_uuid (str): A unique identifier to filter down to only users linked to a
             particular EnterpriseCustomer.
         """
-        learners = EnterpriseCustomerUser.objects.filter(enterprise_customer__uuid=customer_uuid)
-
+        # TODO: Add paging so the admin view does not time out.
+        # For now, we will allow search, but will return an empty list
+        # if no search keyword is provided.
+        learners = []
         if search_keyword is not None:
+            learners = EnterpriseCustomerUser.objects.filter(enterprise_customer__uuid=customer_uuid)
             user_ids = learners.values_list('user_id', flat=True)
             matching_users = User.objects.filter(
                 Q(pk__in=user_ids),
@@ -350,10 +440,12 @@ class EnterpriseCustomerManageLearnersView(View):
                     dict(user=user.username, message=error_message)
                 )
             else:
-                EnterpriseCourseEnrollment.objects.get_or_create(
+                __, created = EnterpriseCourseEnrollment.objects.get_or_create(
                     enterprise_customer_user=enterprise_customer_user,
                     course_id=course_id
                 )
+                if created:
+                    track_enrollment('admin-enrollment', user.id, course_id)
         return succeeded
 
     @classmethod
@@ -373,34 +465,6 @@ class EnterpriseCustomerManageLearnersView(View):
         missing_emails = list(set(emails) - set(present_emails))
         return users, missing_emails
 
-    @classmethod
-    def enroll_user_pending_registration(cls, enterprise_customer, email, course_mode, *course_ids):
-        """
-        Create pending enrollments for the user in any number of courses, which will take effect on registration.
-
-        Args:
-            enterprise_customer: The EnterpriseCustomer which is sponsoring the enrollment
-            email: The email address for the pending link to be created
-            course_mode: The mode with which the eventual enrollment should be created
-            *course_ids: An iterable containing any number of course IDs to eventually enroll the user in.
-
-        Returns:
-            The PendingEnterpriseCustomerUser attached to the email address
-        """
-        pending_ecu, __ = PendingEnterpriseCustomerUser.objects.get_or_create(
-            enterprise_customer=enterprise_customer,
-            user_email=email
-        )
-        for course_id in course_ids:
-            PendingEnrollment.objects.update_or_create(
-                user=pending_ecu,
-                course_id=course_id,
-                defaults={
-                    'course_mode': course_mode
-                }
-            )
-        return pending_ecu
-
     @classmethod
     def enroll_users_in_program(cls, enterprise_customer, program_details, course_mode, emails):
         """
@@ -433,7 +497,11 @@ class EnterpriseCustomerManageLearnersView(View):
                 failures.append(user)
 
         for email in unregistered_emails:
-            pending_user = cls.enroll_user_pending_registration(enterprise_customer, email, course_mode, *course_ids)
+            pending_user = enterprise_customer.enroll_user_pending_registration(
+                email,
+                course_mode,
+                *course_ids
+            )
             pending.append(pending_user)
 
         return successes, pending, failures
@@ -469,7 +537,11 @@ class EnterpriseCustomerManageLearnersView(View):
                 failures.append(user)
 
         for email in unregistered_emails:
-            pending_user = cls.enroll_user_pending_registration(enterprise_customer, email, course_mode, course_id)
+            pending_user = enterprise_customer.enroll_user_pending_registration(
+                email,
+                course_mode,
+                course_id
+            )
             pending.append(pending_user)
 
         return successes, pending, failures
@@ -489,61 +561,6 @@ class EnterpriseCustomerManageLearnersView(View):
             message_function = getattr(messages, msg_type)
             message_function(http_request, text)
 
-    @classmethod
-    def notify_enrolled_learners(cls, enterprise_customer, request, course_id, users):
-        """
-        Notify learners about a course in which they've been enrolled.
-
-        Args:
-            enterprise_customer: The EnterpriseCustomer being linked to
-            request: The HTTP request that's being processed
-            course_id: The specific course the learners were enrolled in
-            users: An iterable of the users or pending users who were enrolled
-        """
-        course_details = CourseCatalogApiClient(request.user).get_course_run(course_id)
-        if not course_details:
-            logging.warning(
-                _(
-                    "Course details were not found for course key {} - Course Catalog API returned nothing. "
-                    "Proceeding with enrollment, but notifications won't be sent"
-                ).format(course_id)
-            )
-            return
-        course_path = urlquote('/courses/{course_id}/course'.format(course_id=course_id))
-        lms_root_url = get_configuration_value_for_site(enterprise_customer.site, 'LMS_ROOT_URL')
-        destination_url = '{site}/{login_or_register}?next={course_path}'.format(
-            site=lms_root_url,
-            login_or_register='{login_or_register}',  # We don't know the value at this time
-            course_path=course_path
-        )
-        course_name = course_details.get('title')
-
-        try:
-            course_start = parse_lms_api_datetime(course_details.get('start'))
-        except (TypeError, ValueError):
-            course_start = None
-            logging.exception(
-                'None or empty value passed as course start date.\nCourse Details:\n{course_details}'.format(
-                    course_details=course_details,
-                )
-            )
-
-        with mail.get_connection() as email_conn:
-            for user in users:
-                login_or_register = 'register' if isinstance(user, PendingEnterpriseCustomerUser) else 'login'
-                destination_url = destination_url.format(login_or_register=login_or_register)
-                send_email_notification_message(
-                    user=user,
-                    enrolled_in={
-                        'name': course_name,
-                        'url': destination_url,
-                        'type': 'course',
-                        'start': course_start,
-                    },
-                    enterprise_customer=enterprise_customer,
-                    email_connection=email_conn
-                )
-
     @classmethod
     def notify_program_learners(cls, enterprise_customer, program_details, users):
         """
@@ -558,8 +575,17 @@ class EnterpriseCustomerManageLearnersView(View):
         program_branding = program_details.get('type')
         program_uuid = program_details.get('uuid')
 
-        lms_root_url = get_configuration_value_for_site(enterprise_customer.site, 'LMS_ROOT_URL')
-        program_path = urlquote('/dashboard/programs/{program_uuid}/'.format(program_uuid=program_uuid))
+        lms_root_url = get_configuration_value_for_site(
+            enterprise_customer.site,
+            'LMS_ROOT_URL',
+            settings.LMS_ROOT_URL
+        )
+        program_path = urlquote(
+            '/dashboard/programs/{program_uuid}/?tpa_hint={tpa_hint}'.format(
+                program_uuid=program_uuid,
+                tpa_hint=enterprise_customer.identity_provider,
+            )
+        )
         destination_url = '{site}/{login_or_register}?next={program_path}'.format(
             site=lms_root_url,
             login_or_register='{login_or_register}',
@@ -695,9 +721,8 @@ class EnterpriseCustomerManageLearnersView(View):
             )
             all_successes = succeeded + pending
             if notify:
-                cls.notify_enrolled_learners(
-                    enterprise_customer=enterprise_customer,
-                    request=request,
+                enterprise_customer.notify_enrolled_learners(
+                    catalog_api_user=request.user,
                     course_id=course_id,
                     users=all_successes,
                 )
@@ -744,7 +769,10 @@ class EnterpriseCustomerManageLearnersView(View):
             django.http.response.HttpResponse: HttpResponse
         """
         context = self._build_context(request, customer_uuid)
-        manage_learners_form = ManageLearnersForm(user=request.user)
+        manage_learners_form = ManageLearnersForm(
+            user=request.user,
+            enterprise_customer=context[self.ContextParameters.ENTERPRISE_CUSTOMER]
+        )
         context.update({self.ContextParameters.MANAGE_LEARNERS_FORM: manage_learners_form})
 
         return render(request, self.template, context)
@@ -761,7 +789,12 @@ class EnterpriseCustomerManageLearnersView(View):
             django.http.response.HttpResponse: HttpResponse
         """
         enterprise_customer = EnterpriseCustomer.objects.get(uuid=customer_uuid)  # pylint: disable=no-member
-        manage_learners_form = ManageLearnersForm(request.POST, request.FILES, user=request.user)
+        manage_learners_form = ManageLearnersForm(
+            request.POST,
+            request.FILES,
+            user=request.user,
+            enterprise_customer=enterprise_customer
+        )
 
         # initial form validation - check that form data is well-formed
         if manage_learners_form.is_valid():
@@ -808,7 +841,11 @@ class EnterpriseCustomerManageLearnersView(View):
                 )
 
             # Redirect to GET if everything went smooth.
-            return HttpResponseRedirect("")
+            manage_learners_url = reverse("admin:" + UrlNames.MANAGE_LEARNERS, args=(customer_uuid,))
+            search_keyword = self.get_search_keyword(request)
+            if search_keyword:
+                manage_learners_url = manage_learners_url + "?q=" + search_keyword
+            return HttpResponseRedirect(manage_learners_url)
 
         # if something went wrong - display bound form on the page
         context = self._build_context(request, customer_uuid)
