@@ -2,15 +2,23 @@
 """
 Utility functions for enterprise app.
 """
-from __future__ import absolute_import, unicode_literals
+from __future__ import absolute_import, division, unicode_literals
 
+import datetime
 import hashlib
 import logging
 import re
 from uuid import UUID
 
+import analytics
+import bleach
+import pytz
+from eventtracking import tracker
+from opaque_keys import InvalidKeyError
 from opaque_keys.edx.keys import CourseKey
 from six import iteritems  # pylint: disable=ungrouped-imports
+# pylint: disable=import-error,wrong-import-order,ungrouped-imports
+from six.moves.urllib.parse import parse_qs, quote_plus, urlencode, urlparse, urlsplit, urlunsplit
 
 from django.apps import apps
 from django.conf import settings
@@ -19,18 +27,23 @@ from django.core.exceptions import ObjectDoesNotExist
 from django.core.urlresolvers import reverse
 from django.http import Http404
 from django.template.loader import render_to_string
+from django.utils.dateparse import parse_datetime
+from django.utils.text import slugify
 from django.utils.translation import ugettext as _
 from django.utils.translation import ungettext
 
-from enterprise.constants import PROGRAM_TYPE_DESCRIPTION
-# pylint: disable=import-error,wrong-import-order,ungrouped-imports
-from six.moves.urllib.parse import parse_qs, urlencode, urlparse, urlsplit, urlunsplit
+from enterprise.constants import ALLOWED_TAGS, PROGRAM_TYPE_DESCRIPTION
 
 try:
     from openedx.core.djangoapps.site_configuration import helpers as configuration_helpers
 except ImportError:
     configuration_helpers = None
 
+try:
+    from lms.djangoapps.branding.api import get_url
+except ImportError:
+    get_url = None
+
 try:
     # Try to import identity provider registry if third_party_auth is present
     from third_party_auth.provider import Registry
@@ -80,16 +93,23 @@ class MultipleProgramMatchError(CourseCatalogApiError):
         self.programs_matched = programs_matched
 
 
+class CourseEnrollmentDowngradeError(Exception):
+    """
+    Exception to raise when an enrollment attempts to enroll the user in an unpaid mode when they are in a paid mode.
+    """
+
+
 def get_identity_provider(provider_id):
     """
     Get Identity Provider with given id.
 
-    Raises a ValueError if it third_party_auth app is not available.
-
     Return:
         Instance of ProviderConfig or None.
     """
-    return Registry and Registry.get(provider_id)
+    try:
+        return Registry and Registry.get(provider_id)
+    except ValueError:
+        return None
 
 
 def get_idp_choices():
@@ -136,17 +156,19 @@ def get_catalog_admin_url(catalog_id):
     return get_catalog_admin_url_template().format(catalog_id=catalog_id)
 
 
-def get_catalog_admin_url_template():
+def get_catalog_admin_url_template(mode='change'):
     """
     Get template of catalog admin url.
 
     URL template will contain a placeholder '{catalog_id}' for catalog id.
+    Arguments:
+        mode e.g. change/add.
 
     Returns:
         A string containing template for catalog url.
 
     Example:
-        >>> get_catalog_admin_url_template()
+        >>> get_catalog_admin_url_template('change')
         "http://localhost:18381/admin/catalogs/catalog/{catalog_id}/change/"
 
     """
@@ -159,7 +181,10 @@ def get_catalog_admin_url_template():
         return ""
 
     # Return matched FQDN from catalog api url appended with catalog admin path
-    return match.group("fqdn").rstrip("/") + "/admin/catalogs/catalog/{catalog_id}/change/"
+    if mode == 'change':
+        return match.group("fqdn").rstrip("/") + "/admin/catalogs/catalog/{catalog_id}/change/"
+    elif mode == 'add':
+        return match.group("fqdn").rstrip("/") + "/admin/catalogs/catalog/add/"
 
 
 def build_notification_message(template_context, template_configuration=None):
@@ -278,18 +303,24 @@ def send_email_notification_message(user, enrolled_in, enterprise_customer, emai
         'organization_name': enterprise_customer.name,
     }
     try:
-        site_template_configuration = enterprise_customer.site.enterprise_enrollment_template
+        enterprise_template_config = enterprise_customer.enterprise_enrollment_template
     except (ObjectDoesNotExist, AttributeError):
-        site_template_configuration = None
+        enterprise_template_config = None
 
-    plain_msg, html_msg = build_notification_message(msg_context, site_template_configuration)
+    plain_msg, html_msg = build_notification_message(msg_context, enterprise_template_config)
 
-    subject_line = get_notification_subject_line(enrolled_in['name'], site_template_configuration)
+    subject_line = get_notification_subject_line(enrolled_in['name'], enterprise_template_config)
+
+    from_email_address = get_configuration_value_for_site(
+        enterprise_customer.site,
+        'DEFAULT_FROM_EMAIL',
+        default=settings.DEFAULT_FROM_EMAIL
+    )
 
     return mail.send_mail(
         subject_line,
         plain_msg,
-        settings.DEFAULT_FROM_EMAIL,
+        from_email_address,
         [user_email],
         html_message=html_msg,
         connection=email_connection
@@ -373,7 +404,7 @@ def get_course_track_selection_url(course_run, query_parameters):
         course_root = reverse('course_modes_choose', kwargs={'course_id': course_run['key']})
     except KeyError:
         LOGGER.exception(
-            "KeyError while parsing course run data.\nCourse Run: \n%s", course_run,
+            "KeyError while parsing course run data.\nCourse Run: \n[%s]", course_run,
         )
         raise
 
@@ -406,25 +437,10 @@ def update_query_parameters(url, query_parameters):
     url_params.update(query_parameters)
 
     return urlunsplit(
-        (scheme, netloc, path, urlencode(url_params, doseq=True), fragment),
+        (scheme, netloc, path, urlencode(sorted(url_params.items()), doseq=True), fragment),
     )
 
 
-def safe_extract_key(data, key, default=''):
-    """
-    Safely extract the key, if it does not exist or is None, return the default.
-
-    Arguments:
-        data (dict): The dictionary from which to extract the key's value.
-        key (str): The key for which we need to extract the value from data.
-        default: The value to return if the key is not present in data.
-
-    """
-    if key in data and data[key] is not None:
-        return data[key]
-    return default
-
-
 def filter_audit_course_modes(enterprise_customer, course_modes):
     """
     Filter audit course modes out if the enterprise customer has not enabled the 'Enable audit enrollment' flag.
@@ -456,24 +472,10 @@ def get_enterprise_customer_or_404(enterprise_uuid):
         enterprise_uuid = UUID(enterprise_uuid)
         return EnterpriseCustomer.objects.get(uuid=enterprise_uuid)  # pylint: disable=no-member
     except (ValueError, EnterpriseCustomer.DoesNotExist):
-        LOGGER.error('Unable to find enterprise customer for UUID: %s', enterprise_uuid)
+        LOGGER.error('Unable to find enterprise customer for UUID: [%s]', enterprise_uuid)
         raise Http404
 
 
-def get_course_id_from_course_run_id(course_run_id):
-    """
-    Given a course run id, return the corresponding course id.
-
-    Args:
-        course_run_id (str): The course run ID.
-
-    Returns:
-        (str): The course ID.
-    """
-    course_run_key = CourseKey.from_string(course_run_id)
-    return '{org}+{course}'.format(org=course_run_key.org, course=course_run_key.course)
-
-
 def clean_html_for_template_rendering(text):
     """
     Given html text that will be rendered as a variable in a template, strip out characters that impact rendering.
@@ -604,7 +606,13 @@ def get_configuration_value_for_site(site, key, default=None):
 def get_configuration_value(val_name, default=None, **kwargs):
     """
     Get a configuration value, or fall back to ``default`` if it doesn't exist.
+
+    Also takes a `type` argument to guide which particular upstream method to use when trying to retrieve a value.
+    Current types include:
+        - `url` to specifically get a URL.
     """
+    if kwargs.get('type') == 'url':
+        return get_url(val_name) or default if callable(get_url) else default
     return configuration_helpers.get_value(val_name, default, **kwargs) if configuration_helpers else default
 
 
@@ -635,3 +643,155 @@ def get_program_type_description(program_type):
     :return: The description associated with the program type. If none exists, then the empty string.
     """
     return PROGRAM_TYPE_DESCRIPTION.get(program_type, '')
+
+
+def get_enterprise_utm_context(enterprise_customer):
+    """
+    Get the UTM context for the enterprise.
+    """
+    return {
+        'utm_medium': 'enterprise',
+        'utm_source': slugify(enterprise_customer.name)
+    }
+
+
+def track_event(user_id, event_name, properties):
+    """
+    Emit a track event to segment (and forwarded to GA) for some parts of the Enterprise workflows.
+    """
+    if getattr(settings, 'LMS_SEGMENT_KEY'):
+        tracking_context = tracker.get_tracker().resolve_context()
+        analytics.track(user_id, event_name, properties, context={
+            'ip': tracking_context.get('ip'),
+            'Google Analytics': {
+                'clientId': tracking_context.get('client_id')
+            }
+        })
+
+
+def track_enrollment(pathway, user_id, course_run_id, url_path=None):
+    """
+    Emit a track event for enterprise course enrollment.
+    """
+    track_event(user_id, 'edx.bi.user.enterprise.onboarding', {
+        'pathway': pathway,
+        'url_path': url_path,
+        'course_run_id': course_run_id,
+    })
+
+
+def parse_datetime_handle_invalid(datetime_value):
+    """
+    Return the parsed version of a datetime string. If the string is invalid, return None.
+    """
+    try:
+        return parse_datetime(datetime_value).replace(tzinfo=pytz.UTC)
+    except TypeError:
+        return None
+
+
+def is_course_run_enrollable(course_run):
+    """
+    Return true if the course run is enrollable, false otherwise.
+
+    We look for the following criteria:
+    - end is greater than now OR null
+    - enrollment_start is less than now OR null
+    - enrollment_end is greater than now OR null
+    """
+    now = datetime.datetime.now(pytz.UTC)
+    end = parse_datetime_handle_invalid(course_run.get('end'))
+    enrollment_start = parse_datetime_handle_invalid(course_run.get('enrollment_start'))
+    enrollment_end = parse_datetime_handle_invalid(course_run.get('enrollment_end'))
+    return (not end or end > now) and \
+           (not enrollment_start or enrollment_start < now) and \
+           (not enrollment_end or enrollment_end > now)
+
+
+def is_course_run_upgradeable(course_run):
+    """
+    Return true if the course run has a verified seat with an unexpired upgrade deadline, false otherwise.
+    """
+    now = datetime.datetime.now(pytz.UTC)
+    for seat in course_run.get('seats', []):
+        if seat.get('type') == 'verified':
+            upgrade_deadline = parse_datetime_handle_invalid(seat.get('upgrade_deadline'))
+            return not upgrade_deadline or upgrade_deadline > now
+    return False
+
+
+def get_course_run_start(course_run, default=None):
+    """
+    Return the given course run's start date as a datetime.
+    """
+    return parse_datetime_handle_invalid(course_run.get('start')) or default
+
+
+def get_current_course_run(course):
+    """
+    Return the current course run.
+
+    Current is defined as the following:
+    - Course run is enrollable (see is_course_run_enrollable)
+    - Course run has a verified seat and the upgrade deadline has not expired.
+    - Course run start date is closer to now than any other enrollable/upgradeable course runs.
+    - If no enrollable/upgradeable course runs, return course run with most recent start date.
+    """
+    now = datetime.datetime.now(pytz.UTC)
+    never = now - datetime.timedelta(days=3650)
+    all_course_runs = course['course_runs']
+    filtered_course_runs = []
+
+    for course_run in all_course_runs:
+        if is_course_run_enrollable(course_run) and is_course_run_upgradeable(course_run):
+            filtered_course_runs.append(course_run)
+
+    if not filtered_course_runs:
+        # Consider all runs if there were not any enrollable/upgradeable ones.
+        filtered_course_runs = all_course_runs
+
+    if filtered_course_runs:
+        # Return course run with start date closest to now.
+        # Course runs with a null start date should be considered last.
+        return min(filtered_course_runs, key=lambda x: abs(get_course_run_start(x, never) - now))
+
+    return None
+
+
+def strip_html_tags(text, allowed_tags=None):
+    """
+    Strip all tags from a string except those tags provided in `allowed_tags` parameter.
+
+    Args:
+        text (str): string to strip html tags from
+        allowed_tags (list): allowed list of html tags
+
+    Returns: a string without html tags
+    """
+    if text is None:
+        return
+    if allowed_tags is None:
+        allowed_tags = ALLOWED_TAGS
+    return bleach.clean(text, tags=allowed_tags, attributes=['id', 'class', 'style', 'href', 'title'], strip=True)
+
+
+def parse_course_key(course_identifier):
+    """
+    Return the serialized course key given either a course run ID or course key.
+    """
+    try:
+        course_run_key = CourseKey.from_string(course_identifier)
+    except InvalidKeyError:
+        # Assume we already have a course key.
+        return course_identifier
+
+    return quote_plus(' '.join([course_run_key.org, course_run_key.course]))
+
+
+def get_content_metadata_item_id(content_metadata_item):
+    """
+    Return the unique identifier given a content metadata item dictionary.
+    """
+    if content_metadata_item['content_type'] == 'program':
+        return content_metadata_item['uuid']
+    return content_metadata_item['key']
